@@ -12,8 +12,8 @@ from time import sleep
 from telegram import ReplyKeyboardMarkup, Bot, KeyboardButton, InlineKeyboardMarkup, InlineKeyboardButton, ParseMode, InputMediaPhoto
 from telegram.ext import Filters
 from .util.filteres import match_btn
-import types
-from telegram import Update
+import types,json
+from telegram import Update,MessageEntity
 
 
 class Base:
@@ -33,7 +33,8 @@ class Base:
         self.back_key = 'back'
         self.back_menu_key = 'menu'
         self.menu_keys = []
-
+        self.lang_code=-1
+        self.text_list=[]
         self.debug: Debug = super_self.debug
         if type(self.debug) == type(None):
             raise Exception('Debug Class Not defined')
@@ -78,14 +79,16 @@ class Base:
             if self.user('role') not in self.role or (self.user('status')!=1 and not self.is_super_admin()):
                 self.status=False
                 return
+            if (self.user('role')!= Role.ADMIN.value and not self.is_super_admin()) and self.get_setting('bot_status')!=1:
+                self.send('bot_off_text',get_text_setting=True)
+                self.status=False
+                return
             forbid_list = ['setter_entry', 'menu_btns']
             if func_name not in forbid_list:
                 self.userdata('last_function_name', func_name)
             self.status = self.run_checkpoint(func_name)
 
     def run_checkpoint(self, func_name):
-        if self.userdata('checkpoint') and self.userdata('checkpoint') == func_name:
-            return True
         for key, func in self.checkpoints.items():
             res = func(self)
             go_on = True
@@ -218,8 +221,10 @@ class Base:
         conversation.arges = {**kargs}
         self.nodes += [conversation]
 
-    def pack(self, pack, repalce_callback_func_name: str = None):
-        pack: Pack = pack(self)
+    def pack(self, pack, repalce_callback_func_name: str = None,**pack_kargs):
+        if pack_kargs:
+            pack: Pack = pack(self,**pack_kargs)
+        else:pack: Pack = pack(self)
         if repalce_callback_func_name:
             for entry in pack.entry:
                 try:
@@ -351,6 +356,12 @@ class Base:
                 return self.update.message.chat.type
         except:
             pass
+    
+    def get_msg_entities(self):
+        return json.dumps([i.to_dict() for i in self.update.effective_message.entities])
+
+    def msg_entities_json_entities_class(self,json_msg_entities):
+        return [MessageEntity(**i) for i in json.loads(json_msg_entities)]
 
     def get_text(self, callback_deny=False):
         try:
@@ -375,16 +386,18 @@ class Base:
             pass
 
     ###################################################
-    def toggle(self, table: str, column: str, id: int, cond_col='id', switch=[0, 1]):
-        res = self.db.do(table, condition=f"{cond_col}={id}")
+    def toggle(self, table: str, column: str, id, cond_col='id', switch=[0, 1]):
+        if isinstance(id,str):
+            cond= f"{cond_col}='{id}'"
+        else:cond= f"{cond_col}={id}"
+        res = self.db.do(table, condition=cond)
         if res:
             new_val = switch[-1]
             if res[0].get(column) == switch[-1]:
                 new_val = switch[0]
-            self.db.do(table, {column: new_val}, condition=f"{cond_col}={id}")
+            self.db.do(table, {column: new_val},cond)
             return new_val
 
-
     def userdata(self, key, value=None, delete=False, regex=False):
         if delete:
             try:
@@ -462,6 +475,8 @@ class Base:
                 'is_online': 1,
                 'last_seen': self.time()
             }
+            if self.is_super_admin():
+                data['role'] =  Role.SUPERADMIN.value
             user = self.db.do('users', condition=f"id={self.id}")
             try:
                 user = user[0]
@@ -506,6 +521,20 @@ class Base:
             pass
         return ''
 
+    def update_text_list(self,force=False):
+        if force or not self.text_list:
+            db_language = self.user('language')
+            lang_cond=''
+            print(db_language)
+            self.lang_code=db_language
+            if db_language==None or db_language==4:db_language=0
+            if db_language!=None:
+                lang_cond = f"language={db_language}"
+            if lang_cond=='':lang_cond=None
+            self.text_list = [i for i in self.db.do('texts', condition=lang_cond) if i['role'] in self.role]
+        return self.text_list
+            
+    
     def text(self, key, return_pair=False, search_by='key', return_id=False, slash=False, **kargs):
         '''
         return pair , target,id
@@ -513,18 +542,17 @@ class Base:
         main_word = kargs.get('main_word')
 
         if key in ['',None]:return key
+        if kargs.get('get_text_setting',False) and (setting_value:=self.get_setting(key)):
+            return setting_value
         if not isinstance(key,str):key=str(key)
         
         if search_by == 'key' and slash and key[0] != '/':
             key = f"/{key}"
-        db_language = self.user('language')
         target = 'key' if search_by == 'key' else 'value'
         pair = 'value' if search_by == 'key' else 'key'
-        lang_cond=''
-        if db_language!=None:
-            lang_cond = f"language={db_language} AND "
         
-        texts = [i for i in self.db.do('texts', condition=f"{lang_cond}{target}='{key}'") if i['role'] in self.role]
+        self.update_text_list(force=(self.lang_code!=self.user('language')))
+        texts = [i for i in self.text_list if i[target]==key]
         if texts:
             text = texts[-1]
             if len(texts)>1 and main_word!=None:
@@ -596,6 +624,7 @@ class Base:
             value, text_key = self.text(key, True,main_word=main_word)
             return value
         else:
+            if not isinstance(key,list):key= list(key)
             for i in range(len(key)):
                 res = self.replace_btn_label(key[i])
                 if res != None:
@@ -609,15 +638,17 @@ class Base:
             reverse:bool=False
             return_raw:bool=False
             add_label:bool=True
+            close=True
 
         '''
         col = args.get('col', 3)
         reverse = args.get('reverse', False)
         add_label = args.get('add_label', True)
         return_raw = args.get('return_raw', False)
-
+        close=args.get('close',True)
         btns = []
         for k, v in inputs.items():
+            k=k.replace('*','')
             key = self.text(k)
             if k.lower() == 'share':
                 btns += [InlineKeyboardButton(key, switch_inline_query=v)]
@@ -631,6 +662,8 @@ class Base:
                 btns += [InlineKeyboardButton(key,
                                               callback_data=val)]
         btns = chunks(btns, col, reverse)
+        if close:
+            btns += [[InlineKeyboardButton('close',callback_data='close')]]   
         if return_raw:
             return btns
         return InlineKeyboardMarkup(btns)
@@ -743,12 +776,16 @@ class Base:
 
             self.send('media_changed')
 
+        self.update_text_list(force=True)
     ###################################################
     def bot_kargs(self, **kargs):
         disable_web_page_preview = kargs.get('disable_web_page_preview')
         reply_to_message_id = kargs.get('reply_to_message_id')
         parse_mode = kargs.get('parse_mode')
         disable_notification = kargs.get('disable_notification')
+        entities = kargs.get('entities')
+        caption_entities = kargs.get('caption_entities')
+        
         out = {}
         if disable_web_page_preview:
             out['disable_web_page_preview'] = disable_web_page_preview
@@ -758,12 +795,17 @@ class Base:
             out['parse_mode'] = parse_mode
         if disable_notification:
             out['disable_notification'] = disable_notification
+        if entities:
+            out['entities'] = entities
+        if caption_entities:
+            out['caption_entities'] = caption_entities
         return out
 
     def send(self, msg, btns=None, chat_id: int = 0, **args):
         '''
-            translat = True
+            translat = True or t=True
             slash=True
+            get_text_setting:bool
 
             col:int
             reply_to_message_id:int,
@@ -773,6 +815,7 @@ class Base:
             disable_web_page_preview:bool=False
             share_phone:bool=False
             reverse_btn:bool=False
+            close:bool =True
 
         '''
 
@@ -806,13 +849,16 @@ class Base:
 
     def send_message(self, msg, btns=None, chat_id: int = 0, **args):
         '''
-            translat = True
+            translat = True or t=True
             slash=True
 
             col:int
             reply_to_message_id:int ,
             parse_mode:PasrMode
             disable_notification
+            entites=[entity]
+            caption_entities=[entity]
+
 
             disable_web_page_preview:bool=False
             share_phone:bool=False
@@ -820,7 +866,7 @@ class Base:
 
         '''
         args['slash'] = args.get('slash', True)
-        args['translat'] = args.get('translat', True)
+        args['translat'] = args.get('translat', True) and args.get('t',True)
         args['disable_web_page_preview'] = args.get(
             'disable_web_page_preview', True)
         btn = self.btn_maker(btns, **args)
@@ -835,15 +881,21 @@ class Base:
             **self.bot_kargs(**args)
         )
         return out['message_id']
-    def send_photo(self, photo:bytes,caption=None, btns=None, chat_id: int = 0, **args):
+    
+    def send_media(self, media:bytes,media_type:str,caption=None, btns=None, chat_id: int = 0, **args):
         '''
-            translat = True
+            media_type:photo,audio,video,document
+            translat = True or t=True
             slash=True
 
             col:int
             reply_to_message_id:int ,
             parse_mode:PasrMode
             disable_notification
+            entites=[entity]
+            caption_entities=[entity]
+
+            
 
             disable_web_page_preview:bool=False
             share_phone:bool=False
@@ -851,7 +903,76 @@ class Base:
 
         '''
         args['slash'] = args.get('slash', True)
-        args['translat'] = args.get('translat', True)
+        args['translat'] = args.get('translat', True) and args.get('t',True)
+        btn = self.btn_maker(btns, **args)
+        if chat_id == 0:
+            chat_id = self.id
+        if args.get('translat'):
+            caption = self.text(caption, **args)
+        if media_type=="photo":
+            out = self.context.bot.send_photo(
+                photo = media,
+                chat_id=chat_id,
+                caption= caption,
+                reply_markup=btn,
+                **self.bot_kargs(**args)
+            )
+        elif media_type=="animation":
+            out = self.context.bot.send_animation(
+                animation = media,
+                chat_id=chat_id,
+                caption= caption,
+                reply_markup=btn,
+                **self.bot_kargs(**args)
+            )
+        elif media_type=="video":
+            out = self.context.bot.send_video(
+                video = media,
+                chat_id=chat_id,
+                caption= caption,
+                reply_markup=btn,
+                **self.bot_kargs(**args)
+            )
+        elif media_type=="audio":
+            out = self.context.bot.send_audio(
+                audio = media,
+                chat_id=chat_id,
+                caption= caption,
+                reply_markup=btn,
+                **self.bot_kargs(**args)
+            )
+        elif media_type=="document":
+            out = self.context.bot.send_document(
+                document = media,
+                chat_id=chat_id,
+                caption= caption,
+                reply_markup=btn,
+                **self.bot_kargs(**args)
+            )
+        return out['message_id']
+
+    
+
+    def send_animation(self, photo:bytes,caption=None, btns=None, chat_id: int = 0, **args):
+        '''
+            translat = True or t=True
+            slash=True
+
+            col:int
+            reply_to_message_id:int ,
+            parse_mode:PasrMode
+            disable_notification
+            entites=[entity]
+            caption_entities=[entity]
+
+
+            disable_web_page_preview:bool=False
+            share_phone:bool=False
+            reverse_btn:bool=False
+
+        '''
+        args['slash'] = args.get('slash', True)
+        args['translat'] = args.get('translat', True) and args.get('t',True)
         btn = self.btn_maker(btns, **args)
         if chat_id == 0:
             chat_id = self.id
@@ -869,12 +990,15 @@ class Base:
     
     def copy_message(self, from_chat_id, message_id, btns=None, chat_id=0, caption=None, **args):
         '''
-            translat = True
+            translat = True or t=True
             slash=True
             col:int
             reply_to_message_id:int ,
             parse_mode:PasrMode
             disable_notification
+            entites=[entity]
+            caption_entities=[entity]
+
 
             disable_web_page_preview:bool=False
             share_phone:bool=False
@@ -882,7 +1006,7 @@ class Base:
 
         '''
         args['slash'] = args.get('slash', True)
-        args['translat'] = args.get('translat', True)
+        args['translat'] = args.get('translat', True) and args.get('t',True)
         btn = self.btn_maker(btns, **args)
         if chat_id == 0:
             chat_id = self.id
@@ -900,12 +1024,15 @@ class Base:
 
     def edit_message_text(self, new_msg, message_id, btns=None, chat_id: int = 0, **args):
         '''
-            translat = True
+            translat = True or t=True
             slash=True
             col:int
             reply_to_message_id:int ,
             parse_mode:PasrMode
             disable_notification
+            entites=[entity]
+            caption_entities=[entity]
+
 
             disable_web_page_preview:bool=False
             share_phone:bool=False
@@ -913,7 +1040,7 @@ class Base:
 
         '''
         args['slash'] = args.get('slash', True)
-        args['translat'] = args.get('translat', True)
+        args['translat'] = args.get('translat', True) and args.get('t',True)
         try:
             btn = self.btn_maker(btns, **args)
             if chat_id == 0:
@@ -934,13 +1061,16 @@ class Base:
 
     def edit_message_caption(self, new_caption, msg_id, btns=None, chat_id: int = 0, **args):
         '''
-            translat = True
+            translat = True or t=True
             slash=True
 
             col:int
             reply_to_message_id:int ,
             parse_mode:PasrMode
             disable_notification
+            entites=[entity]
+            caption_entities=[entity]
+
 
             disable_web_page_preview:bool=False
             share_phone:bool=False
@@ -948,7 +1078,7 @@ class Base:
 
         '''
         args['slash'] = args.get('slash', True)
-        args['translat'] = args.get('translat', True)
+        args['translat'] = args.get('translat', True) and args.get('t',True)
         try:
             btn = self.btn_maker(btns, **args)
             if chat_id == 0:
@@ -973,6 +1103,9 @@ class Base:
             reply_to_message_id:int ,
             parse_mode:PasrMode
             disable_notification
+            entites=[entity]
+            caption_entities=[entity]
+
 
             disable_web_page_preview:bool=False
             share_phone:bool=False
@@ -1001,6 +1134,9 @@ class Base:
             reply_to_message_id:int ,
             parse_mode:PasrMode
             disable_notification
+            entites=[entity]
+            caption_entities=[entity]
+
 
             disable_web_page_preview:bool=False
             share_phone:bool=False
@@ -1139,7 +1275,7 @@ class Base:
         return msg_id
         
 
-    def pagination_maker(self, items: list, per_page=5, msg_list=False):
+    def pagination_maker(self, items: list, per_page=5, msg_list=False,revers_list=False):
         '''
         return msg, btn
 
@@ -1147,6 +1283,8 @@ class Base:
         items = chunks(list(reversed(items)), per_page)
         self.userdata('pagination_items', items)
         self.userdata('pagination_iter', 0)
+        self.userdata('pagination_revers_list', revers_list)
+        
         try:
             msg = self.pagination_msg_maker(msg_list)
             btn = self.pagination_btn_maker()
@@ -1159,7 +1297,9 @@ class Base:
     def pagination_msg_maker(self, msg_list=False):
         items = self.userdata('pagination_items')
         idx = self.userdata('pagination_iter')
-        item = list(reversed(items[idx]))
+        if not self.userdata('pagination_revers_list'):
+            item = list(reversed(items[idx]))
+        else:item = list(items[idx])
         if msg_list:
             return item
         msg = ''
@@ -1196,6 +1336,25 @@ class Base:
             return False
         return checker
 
+    def force_join_inline_filter(self):
+        def checker(*args):
+            try:
+                channels = self.db.do('force_join')
+                if len(channels) > 0:
+                    for ch in channels:
+                        channel_id = ch['channel_id']
+                        try:
+                            if not self.is_channel_member(channel_id):
+                                return True
+                        except Exception as e:
+                            pass
+            except Exception as e:print(e)
+            return False
+        return checker
+    
+
+
+
     def is_channel_member(self, channel_id, user_id=0):
         if user_id == 0:
             user_id = self.id
